#region usings
using System;
using System.ComponentModel.Composition;

using VVVV.PluginInterfaces.V1;
using VVVV.PluginInterfaces.V2;
using VVVV.Utils.VColor;
using VVVV.Utils.VMath;
#endregion usings

namespace VVVV.Nodes
{
	public struct Triangle
	{
		public Vector3D A;
		public Vector3D B;
		public Vector3D C;
		public Vector3D n;

		public Triangle(Vector3D a, Vector3D b, Vector3D c)
		{
			A = a;
			B = b;
			C = c;
			n = (A - B) & (A - C);
		}
	}

	#region PluginInfo
	[PluginInfo(Name = "GenerateEquations", Category = "3d", Version = "Triangles", Tags = "affine transform")]
	#endregion PluginInfo
	public class Triangles3dGenerateEquationsNode : IPluginEvaluate
	{
		#region fields & pins
		[Input("Source Triangle ")]
		ISpread<Vector3D> FSrcTri;

		[Input("Destination Triangle ")]
		ISpread<Vector3D> FDestTri;

		[Output("A")]
		ISpread<double> FAOut;

		[Output("b")]
		ISpread<double> FbOut;

		#endregion fields & pins

		public void Evaluate(int SpreadMax)
		{
			SpreadMax = (int)Math.Ceiling(SpreadMax / 3.0);
			FAOut.SliceCount = SpreadMax * 12 * 12;
			FbOut.SliceCount = SpreadMax * 12;

			for (int i = 0; i < SpreadMax; i++) {
				int t = i * 3;
				Triangle src = new Triangle(FSrcTri[t], FSrcTri[t + 1], FSrcTri[t + 2]);
				Triangle dest = new Triangle(FDestTri[t], FDestTri[t + 1], FDestTri[t + 2]);

				SetAOut(src, i);

				int b = i * 12;
				FbOut[b + 0] = dest.A.x;
				FbOut[b + 1] = dest.B.x;
				FbOut[b + 2] = dest.C.x;
				FbOut[b + 3] = dest.n.x;
				FbOut[b + 4] = dest.A.y;
				FbOut[b + 5] = dest.B.y;
				FbOut[b + 6] = dest.C.y;
				FbOut[b + 7] = dest.n.y;
				FbOut[b + 8] = dest.A.z;
				FbOut[b + 9] = dest.B.z;
				FbOut[b + 10] = dest.C.z;
				FbOut[b + 11] = dest.n.z;
			}

		}

		private void SetAOut(Triangle src, int index)
		{
			int a = index * 144;

			//block 1
			FAOut[a] = src.A.x;
			FAOut[a + 1] = src.A.y;
			FAOut[a + 2] = src.A.z;
			FAOut[a + 3] = 1;

			a += 12;
			FAOut[a] = src.B.x;
			FAOut[a + 1] = src.B.y;
			FAOut[a + 2] = src.B.z;
			FAOut[a + 3] = 1;

			a += 12;
			FAOut[a] = src.C.x;
			FAOut[a + 1] = src.C.y;
			FAOut[a + 2] = src.C.z;
			FAOut[a + 3] = 1;

			a += 12;
			FAOut[a] = src.n.x;
			FAOut[a + 1] = src.n.y;
			FAOut[a + 2] = src.n.z;
			FAOut[a + 3] = 0;

			//block 2
			a += 16;
			FAOut[a] = src.A.x;
			FAOut[a + 1] = src.A.y;
			FAOut[a + 2] = src.A.z;
			FAOut[a + 3] = 1;

			a += 12;
			FAOut[a] = src.B.x;
			FAOut[a + 1] = src.B.y;
			FAOut[a + 2] = src.B.z;
			FAOut[a + 3] = 1;

			a += 12;
			FAOut[a] = src.C.x;
			FAOut[a + 1] = src.C.y;
			FAOut[a + 2] = src.C.z;
			FAOut[a + 3] = 1;

			a += 12;
			FAOut[a] = src.n.x;
			FAOut[a + 1] = src.n.y;
			FAOut[a + 2] = src.n.z;
			FAOut[a + 3] = 0;

			//block 3
			a += 16;
			FAOut[a] = src.A.x;
			FAOut[a + 1] = src.A.y;
			FAOut[a + 2] = src.A.z;
			FAOut[a + 3] = 1;

			a += 12;
			FAOut[a] = src.B.x;
			FAOut[a + 1] = src.B.y;
			FAOut[a + 2] = src.B.z;
			FAOut[a + 3] = 1;

			a += 12;
			FAOut[a] = src.C.x;
			FAOut[a + 1] = src.C.y;
			FAOut[a + 2] = src.C.z;
			FAOut[a + 3] = 1;

			a += 12;
			FAOut[a] = src.n.x;
			FAOut[a + 1] = src.n.y;
			FAOut[a + 2] = src.n.z;
			FAOut[a + 3] = 0;
		}
	}


}
